function [C, L, U] = spectclust(A, k, varargin)
%SPECTCLUST Performs spectral clustering with asymmetric Laplacian
%  C = SPECTCLUST(A, k) clusters adjacency matrix A into k clusters and
%  returns the cluster assignment vector C.
%
%  SPECTCLUST(A, k, param1, param2, ...) will pass the extra parameters
%  to k-means.
%
%  [C, L, U] = SPECTCLUST(A, k) returns also the Laplacian L and k smallest
%  eigenvectors (excl. the smallest) in U.
%
%  See also: KMEANS

%% Check that the input is valid
p = inputParser;
addRequired(p, 'A', @(x) validateattributes(x, {'numeric'}, {'square', ...
                    'binary'}, mfilename, 'A', 1));
addRequired(p, 'k', @(x) validateattributes(x, {'numeric'}, {'scalar', ...
                    'positive'}, mfilename, 'A', 1));

parse(p, A, k);

N = size(A, 1);

%% Compute the Laplacian
degs = sum(A, 2);
degs(degs == 0) = eps; % Avoid dividing with 0
D = spdiags(degs, 0, N, N);
L = D - A; % The unnormalized Laplacian
D = spdiags(1./degs, 0, N, N); % D^{-1}
L = D * L; % The normalized Laplacian

%% Compute the k+1 smallest eigenvectors and cluster all but the last of
%% them
[U, ~] = eigs(L, k+1, 'sm');
U = U(:, 1:k);
C = kmeans(U, k, varargin{:});


